//
//  AppDelegate.m
//  YYObjCDemo
//
//  Created by Daniel Bey on 2017年06月16日.
//  Copyright © 2017 百度鹰眼. All rights reserved.
//

#import "AppDelegate.h"
#import "YYMainViewController.h"

@interface AppDelegate ()
@end

@implementation AppDelegate

#pragma mark - Application Lifecycle
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {

    // 申请通知权限
    if (CURRENT_IOS_VERSION >= 10.0) {
        [[UNUserNotificationCenter currentNotificationCenter] requestAuthorizationWithOptions:(UIUserNotificationTypeBadge | UIUserNotificationTypeAlert | UIUserNotificationTypeSound) completionHandler:^(BOOL granted, NSError * _Nullable error) {
            if (granted) {
                NSLog(@"已经获取通知权限");
            }
        }];
    }
    
    // 注册通知
    [UNUserNotificationCenter currentNotificationCenter].delegate = self.notificationHandler;
    
    // 设置鹰眼SDK的基础信息
    // 每次调用startService开启轨迹服务之前，可以重新设置这些信息。
    BTKServiceOption *basicInfoOption = [[BTKServiceOption alloc] initWithAK:AK mcode:MCODE serviceID:serviceID keepAlive:FALSE];
    [[BTKAction sharedInstance] initInfo:basicInfoOption];
    
    // 初始化地图SDK
    BMKMapManager *mapManager = [[BMKMapManager alloc] init];
    [mapManager start:AK generalDelegate:self];
    
    // 加载DEMO的主页面
    YYMainViewController *rootVC = [[YYMainViewController alloc] init];
    UINavigationController *navigationController = [[UINavigationController alloc] initWithRootViewController:rootVC];
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window.rootViewController = navigationController;
    [self.window makeKeyAndVisible];
    return YES;
}

-(void)applicationDidBecomeActive:(UIApplication *)application {
    // 每次进入应用时将角标清零
    [UIApplication sharedApplication].applicationIconBadgeNumber = 0;
}

#pragma mark - BMKGeneralDelegate
-(void)onGetNetworkState:(int)iError {
    if (0 == iError) {
        NSLog(@"联网成功");
    } else{
        NSLog(@"onGetNetworkState %d",iError);
    }
}

- (void)onGetPermissionState:(int)iError {
    if (0 == iError) {
        NSLog(@"授权成功");
    } else {
        NSLog(@"onGetPermissionState %d",iError);
    }
}

#pragma mark - setter & getter
-(YYNotificationHandler *)notificationHandler {
    if (_notificationHandler == nil) {
        _notificationHandler = [[YYNotificationHandler alloc] init];
    }
    return _notificationHandler;
}

@end




