//
//  YYRootViewController.m
//  YYObjCDemo
//
//  Created by Daniel Bey on 2017年06月16日.
//  Copyright © 2017 百度鹰眼. All rights reserved.
//

#import "YYMainViewController.h"

@interface YYMainViewController ()
@property (nonatomic, copy) NSArray *functionList;
@end

@implementation YYMainViewController

#pragma mark - life cycle
- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupUI];
}

#pragma mark - private function
- (void)setupUI {
    self.title = [NSString stringWithFormat:@"鹰眼SDK v%@ DEMO", BTKVersionNumber];
    UIBarButtonItem *backBarButtonItem = [[UIBarButtonItem alloc] init];
    backBarButtonItem.title = @"返回";
    self.navigationItem.backBarButtonItem = backBarButtonItem;
    self.tableView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
    self.tableView.separatorInset = UIEdgeInsetsZero;
}

#pragma mark - UITableViewDelegate
-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.functionList.count;
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"YYFunctionListCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        cell.textLabel.font = [UIFont boldSystemFontOfSize:24];
    }
    cell.imageView.image = [UIImage imageNamed:self.functionList[indexPath.row][@"icon"]];
    CGSize itemSize = CGSizeMake(40, 40);
    UIGraphicsBeginImageContextWithOptions(itemSize, NO, UIScreen.mainScreen.scale);
    CGRect imageRect = CGRectMake(0.0, 0.0, itemSize.width, itemSize.height);
    [cell.imageView.image drawInRect:imageRect];
    cell.imageView.image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    cell.textLabel.text = self.functionList[indexPath.row][@"title"];
    cell.detailTextLabel.text = self.functionList[indexPath.row][@"detail"];
    return cell;
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 60;
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    NSString *className = self.functionList[indexPath.row][@"className"];
    UIViewController *subVC = [[NSClassFromString(className) alloc] init];
    subVC.title = self.functionList[indexPath.row][@"title"];
    [self.navigationController pushViewController:subVC animated:NO];
}

#pragma mark - getter & setter
-(NSArray *)functionList {
    if (_functionList == nil) {
        NSString *name = [MAIN_BUNDLE pathForResource:@"functionName.json" ofType:nil];
        NSData *data = [NSData dataWithContentsOfFile:name];
        _functionList = [NSJSONSerialization JSONObjectWithData:data options:0 error:nil];
    }
    return _functionList;
}

@end
